from box import BoxKeyError
from pydantic import Field, HttpUrl
from typing import Type

from datagen_protocol.schema import d3 as core_3d_schema


def numeric(field_config) -> Field:
    field_params = _parse_common_params(field_config)
    try:
        field_params["le"] = field_config.max
    except BoxKeyError:
        pass
    try:
        field_params["ge"] = field_config.min
    except BoxKeyError:
        pass
    return Field(**field_params)


def enum(enum_type: Type, field_config) -> Field:
    field_params = _parse_common_params(field_config)
    try:
        default_val_str = field_params["default"]
        field_params["default"] = enum_type[default_val_str]
    except KeyError:
        pass
    return Field(**field_params)


def bool(field_config) -> Field:
    field_params = _parse_common_params(field_config)
    return Field(**field_params)


def point(field_config) -> Field:
    return Field(
        default_factory=lambda: core_3d_schema.Point(
            x=field_config.x.default, y=field_config.y.default, z=field_config.z.default
        )
    )


def rotation(field_config) -> Field:
    return Field(
        default_factory=lambda: core_3d_schema.Rotation(
            yaw=field_config.yaw.default, pitch=field_config.pitch.default, roll=field_config.roll.default
        )
    )


def vector(field_config) -> Field:
    return Field(
        default_factory=lambda: core_3d_schema.Vector(
            x=field_config.x.default, y=field_config.y.default, z=field_config.z.default
        )
    )


def _parse_common_params(field_config) -> dict:
    field_params = {}
    try:
        field_params["title"] = field_config.title
    except BoxKeyError:
        pass
    try:
        field_params["description"] = field_config.description
    except BoxKeyError:
        pass
    try:
        field_params["default"] = field_config.default
    except BoxKeyError:
        pass
    return field_params



from enum import Enum
from typing import Union, List, TypeVar, Generic

from pydantic import BaseModel, Extra
from pydantic.generics import GenericModel
from pydantic.fields import ModelField, Field


Attribute = TypeVar("Attribute")

AssetAttributes_ = TypeVar("AssetAttributes_")

AssetPresets_ = TypeVar("AssetPresets_")


class SchemaBaseModel(BaseModel):
    class Config:
        extra = Extra.forbid


class AttributesList(list, Generic[Attribute]):
    @classmethod
    def __get_validators__(cls):
        yield cls.validate

    @classmethod
    def validate(cls, v: Union[Attribute, List[Attribute]], field: ModelField) -> List[Attribute]:
        if not isinstance(v, list):
            v = [v]
        return [field.type_(v_) for v_ in v]


class AssetAttributes(BaseModel):
    class Config:
        frozen = True


class Asset(SchemaBaseModel, GenericModel, Generic[AssetAttributes_]):
    id: str
    attributes: AssetAttributes_ = Field(default=None)

    def dict(self, *args, **kwargs):
        kwargs["exclude"] = {"attributes"}
        return super().dict(*args, **kwargs)


class PresetAsset(SchemaBaseModel, GenericModel, Generic[AssetAttributes_, AssetPresets_]):
    id: str
    attributes: AssetAttributes_ = Field(default=None)
    presets: AssetPresets_ = Field(default=None)

    def dict(self, *args, **kwargs):
        kwargs["exclude"] = {"attributes", "presets"}
        return super().dict(*args, **kwargs)



from typing import List, Union

from datagen_protocol import __version__ as __datagen_protocol_version__
from datagen_protocol.schema.base import SchemaBaseModel
from datagen_protocol.schema.hic.sequence import DataSequence
from datagen_protocol.schema.humans import HumanDatapoint


class DataRequest(SchemaBaseModel):
    __protocol_version__: str = __datagen_protocol_version__

    datapoints: List[Union[DataSequence, HumanDatapoint]]

    def __len__(self):
        return len(self.datapoints)



from .camera import (
    Camera,
    SequenceCamera,
    IntrinsicParams,
    ExtrinsicParams,
    Wavelength,
    Projection,
)
from .background import Background
from .light import Light, LightType



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['datagen_protocol',
 'datagen_protocol.functional',
 'datagen_protocol.schema',
 'datagen_protocol.schema.environment',
 'datagen_protocol.schema.hic',
 'datagen_protocol.schema.humans']

package_data = \
{'': ['*'],
 'datagen_protocol': ['resources/*', 'resources/expressions_presets/*']}

install_requires = \
['dynaconf>=2.2.3,<3.0.0', 'numpy>=1.24.1,<2.0.0', 'pydantic>=1.10.4,<2.0.0']

setup_kwargs = {
    'name': 'datagen-protocol-functional',
    'version': '0.7.0',
    'description': 'Datagen Functional Protocol',
    'long_description': 'None',
    'author': 'ShayZ',
    'author_email': 'shay.zilberman@datagen.tech',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from typing import Optional

from pydantic import BaseModel, Field

from datagen_protocol.config import conf
from datagen_protocol.schema import fields
from datagen_protocol.schema.base import AssetAttributes, Asset
from datagen_protocol.schema.attributes import Environment, TimeOfDay, Generator


class BackgroundAttributes(AssetAttributes):
    environment: Environment
    time_of_day: TimeOfDay
    generator: Generator


class Background(Asset[BackgroundAttributes]):
    rotation: float = fields.numeric(conf["background"]["rotation"])
    transparent: bool = fields.bool(conf["background"]["transparency"])



import abc
from typing import TypeVar, Type

from pydantic import Field

from datagen_protocol.config import conf
from datagen_protocol.schema.base import SchemaBaseModel


class Coords3D(SchemaBaseModel):
    x: float = Field(ge=conf["d3"]["boundaries"]["min"], le=conf["d3"]["boundaries"]["max"])
    y: float = Field(ge=conf["d3"]["boundaries"]["min"], le=conf["d3"]["boundaries"]["max"])
    z: float = Field(ge=conf["d3"]["boundaries"]["min"], le=conf["d3"]["boundaries"]["max"])


class Point(Coords3D):
    pass


class Vector(Coords3D):
    pass


class Rotation(SchemaBaseModel):
    yaw: float
    roll: float
    pitch: float
